#!/usr/bin/python

# Function executes three simulations, creating output directories with scheme:
# <project_name_without_extension>-<cpu-count>-<run number>
# If directory exists:
# - if summary.txt file is present, run is skipped
# - summary.txt does not exist -> simulation is redone
# once all runs are complete, the summary.txt files are read and the best one is selected and returned

import os
import subprocess
import platform
import SolverStats

def run_best_of_N(cmdline, baseDir, projectPath, outputBaseDir, N):
	"""Runs project several times, collects statistics and return stats of fastest run.

	Arguments:

	- cmdline - Array with command line arguments excluding the project file
	- baseDir - path to root of performance test suite
	- projectPath - path to project file relative to baseDir
	- outputBaseDir - path to output dir
	- N - number of runs

	"""
	
	if not os.path.exists(cmdline[0]):
		raise RuntimeError("Solver executable not found!")

	if not os.path.exists(outputBaseDir):
		os.mkdir(outputBaseDir)

	# check for -p=xxx parameter and extract number as optional suffix to output path
	threadCountSuffix = 0
	for arg in cmdline:
		p = arg.find("-p=")
		if p != -1:
			threadCountSuffix=int(arg[p+3:])
	if threadCountSuffix == 0:
		print "Missing number of threads argument in command line, assuming sequentiel run and setting -p=1 explicitely."
		cmdline += ["-p=1"]
		threadCountSuffix = 1

	# loop over all runs
	for i in range(N):
		print ("Run #{}".format(i+1))
		# construct full path to project
		project = os.path.join(baseDir, projectPath)
		# construct target dir
		d = os.path.join(outputBaseDir, project[:-4] + '-{:02d}-{}'.format(threadCountSuffix, i+1))
		# directory exists
		if os.path.exists(d):
			# summary.txt exists
			if os.path.exists( os.path.join(d, 'log/summary.txt')):
				print "Run #{} already completed, skipped".format(i+1)
				continue

		completeCmdLine = cmdline + [project, '-o='+d]
		# run solver
		try:
			if platform.system() == "Windows":
				solverProcess = subprocess.Popen(completeCmdLine, creationflags=subprocess.CREATE_NEW_CONSOLE)
			else:
				solverProcess = subprocess.Popen(completeCmdLine)
			solverProcess.wait()
			s = solverProcess.communicate()
		except OSError as err:
			print err
			exit (1)

	# loop again over all runs, extract summary.txt data and generate best-of-x data
	bestRun = None
	for i in range(N):
		# construct target directory
		d = project[:-4] + '-{:02d}-{}'.format(threadCountSuffix, i+1)
		summaryFile = os.path.join(d, 'log/summary.txt')
		# file exists?
		if not os.path.exists(summaryFile):
			print 'Missing summary.txt file for test run {}'.format(i+1)
			continue
		s = SolverStats.SolverStats()
		s.read(summaryFile)
		print "Run #{}: WCT = {} s".format(i+1, s.timers["WallClockTime"])
		if bestRun == None:
			bestRun = s
		else:
			if float(bestRun.timers["WallClockTime"]) > float(s.timers["WallClockTime"]):
				bestRun = s

	return bestRun
		
		
	

# *** Test script ***

# generate test command line
#cmdline = ['/home/ghorwin/svn/Delphin6_trunk/bin/release/DelphinSolver', '-p=4']
#baseDir = '/home/ghorwin/Schreibtisch/2018_DELPHINParallelPerformanceTests'
#projectPath = '2D_Ecke/Ecke_autodisc.d6p'
#outputRootPath = '/home/ghorwin/tmp/parTests'

## get best out of 3 runs
#results = run_best_of_N(cmdline, baseDir, projectPath, outputRootPath, 3)

