#!/usr/bin/python

# Syntax:
#   runPerformanceTests.py <test-case-root-dir> <thread_variants> <output-root-dir> [-n <best-of-N>]
#
# Example: run with 1, 2 and 4 threads
#
# > runPerformanceTests.py /home/username/performanceTests 1|2|4 /var/tmp/perfTests
#

import os
import os.path
import argparse
import sys
import platform

from best_of_N import run_best_of_N

FILE_EXTENSION = "d6p"
SOLVER_EXECUTABLE = "bin/linux64/DelphinSolver"

HELPTEXT = """
Syntax: runPerformanceTests.py <test-case-root-dir> <thread_variants> [<output-root-dir>] [-n <best-of-N>]

The script will recursively scan the directory <test-case-root-dir> and its subdirs for
simulation projects and run those several times and store the statistics for the fastest run.

The thread variants are | separated list of positive integers

Example:
> runPerformanceTests.py /home/username/performanceTests 1|2|4 /var/tmp/perfTests

"""


# *** main ***

# setup command line parser
parser = argparse.ArgumentParser("runPerformanceTests.py", epilog=HELPTEXT)
parser.add_argument('projectRoot', help="The test case project base directory")
parser.add_argument('threadVariants', nargs='?', help="Encoded string of variants to run with different number of CPUs", default="1|4")
parser.add_argument('outputRootDir', nargs='?', help="The base path for writing result files.")
parser.add_argument('-n', action='store', type=int, default=3, dest="runCount", help="Number of runs to use")

args = parser.parse_args()

try:
    threadVariants = [int(cpuCount) for cpuCount in args.threadVariants.split('|')]
except:
    print("Invalid format of threadVariants argument.")
    sys.exit(1)

outputRootDir = args.outputRootDir
if outputRootDir == None:
    outputRootDir = os.path.abspath(args.projectRoot)

if platform.system() == "Windows":
    SOLVER_EXECUTABLE = "bin/win64/DelphinSolver.exe"

projectRoot = os.path.abspath(args.projectRoot)

print "Processing base directory '{}'".format(projectRoot)
print "  Outputs are written in '{}'".format(outputRootDir)
print "  CPU variants '{}'".format(args.threadVariants)
print "  Number of runs per case '{}'\n".format(args.runCount)

# loop over thread variants



for threadCount in threadVariants:
    print "--- Cases with #{} thread(s) ---".format(threadCount)

    default_cmdline = [ os.path.join(projectRoot, SOLVER_EXECUTABLE), '-x', '--verbosity-level=0']

    results = dict()

    for (dirpath, dirnames, filenames) in os.walk(projectRoot):
        for f in filenames:
            if len(f)>4 and f[-4:]==".d6p":
                projectPath = os.path.join(dirpath,f)
                projectPath = os.path.relpath(projectPath, projectRoot)
                print '{}'.format(projectPath)
                cmdline = default_cmdline + ['-p={}'.format(threadCount)]
                bestStats = run_best_of_N(cmdline, projectRoot, projectPath, outputRootDir, args.runCount)
                results[projectPath] = bestStats

    # create result file
    resPath = os.path.join(args.projectRoot, "results-{:02d}.csv".format(threadCount))
    fobj = open(resPath, 'w')
    fobj.write("Case results for runs with #{} threads, and using best-of-{}\n".format(threadCount,args.runCount))
    
    # collect all time-based keywords
    #timeKeywords = set()
    #for case in results:
        #for keyword in results[case].timers:
            #timeKeywords.add(keyword)
    
    timeKeywords = ['WallClockTime',
                    'FrameworkTimeWriteOutputs',
                    'IntegratorTimeFunctionEvals',
                    'LESTimeRHSEvals',
                    'LESTimeRHSEvalsJacGeneration',
                    'IntegratorTimeLESSetup',
                    'IntegratorTimeLESSolve']
            
    # now write all timers to a data table
    fobj.write("Stats\t")
    for case in results:
        fobj.write("{}\t".format(case))
    fobj.write("\n")
    
    for k in timeKeywords:
        fobj.write(k + '\t')
        for case in results:
            if k in results[case].timers:
                fobj.write("{}\t".format(results[case].timers[k]))
            else:
                fobj.write("---\t")
        fobj.write("\n")
        
    fobj.close()
    del fobj
